; Paced Loop Framework Program
; Assembled with CASM05

#INCLUDE 'H705J1A.FRK'        ; Include the equates for the 
					; HC705J1A so all labels can be found. 

;  Application specific equates
LED		EQU	PA7	; LED ON when PA7 is low (0)
LED.		EQU	PA7.	; LED bit position
SW		EQU	PA0	; Switch on PA0, closed=high (1)
SW.		EQU	PA0.	; Switch bit position

;************************************************************************************
; Put program variables here (use RMBs)
;************************************************************************************
		ORG	RAM	; Start of 705J1A RAM
RTIFs		RMB	1	; 3 RTIFs/TIC (3-0)
TIC		RMB	1	; 10 TICs make 1 TOC (10-0)
				; MSB=1 means RTIFs roller over
TOC		RMB	1	; 1 TOC=10*96.24ms=about 1 sec

;*************************************************************************************
; Program area starts here
;*************************************************************************************
		ORG	$0300	; Start of 705J1A EPROM

; First initialize any control registers and variables.

START		CLI		; Clear I bit for interrupts
		LDA	#LED.	; Configure and turn off LED
		STA	PORTA	; Turn off LED
		STA	DDRA	; Makes LED pin an output
		LDA	#{RTIFR.+RTIE.+RT1.}
		STA	TSCR	; To clear and enable RTIF and set RTI rate for 32.8ms
		LDA	#3	; RTIFs count 3 --> 0
		STA	RTIFs	; Reset TOFS count
		CLR	TIC	; Initial value for TIC
		CLR	TOC	; Initial value for TOC

;*************************************************************************************
;  MAIN - Beginning of main program loop.  Loop is executed once every 100 ms (98.4 ms).  A pass
; though all major task routines takes less than 100ms and then time is wasted until MSB of TIC set 
; (every 3 RTIFs =98.4 ms).  At each RTIF interrupt, RTIF cleared & RTIFs gets decremented (3-0).
; When RTIFs = 0, MSB of TIC gets set and RTIFs is set back to 3. (332,8/RTIF = 98.4 ms).
;
; The variable TIC keeps track of 100ms periods.  When TIC increments from 9 to 10, it is cleared to
; 0 and TOC is incremented.
;*************************************************************************************
MAIN		CLRA		; Kick the watch dog
		STA	COPR	; if enabled
		BRCLR	7,TIC,MAIN	; Loop here until TIC edge
		LDA	TIC		; Get current TIC value
		AND	#$0F		; Clears MSB
		INCA			; TIC = TIC + 1
		STA	TIC		; Update TIC
		CMP	#10		; 10th TIC?
		BNE	ARNC1		; If not, skip next clear
		CLR	TIC		; Clear TIC on 10th
ARNC1		EQU	*

; End of synchronization to 100ms.  Sync OK as long as no 2 consecutive passes take more than 196.8ms

		JSR	TIME		; Update TOCs
		JSR	BLINK		; Blink LED

; Other main tasks would go here
		BRA	MAIN		; Back to Top for next TIC

;******************** END of MAIN LOOP **********************************************

;*************************************************************************************
; TIME - Update TOCs
;  If TIC = 0, increment 0 --> 59
; IF TIC not = 0, just skip whole routine
;*************************************************************************************
TIME		EQU	*		; Update TOCs
		TST	TIC		; Check for TIC = 0
		BNE	XTIME		; If not, just exit
		INC	TOC		; Increment TOC
		LDA	#60		; 
		CMP	TOC		; Did TOC --> 60?
		BNE	XTIME		;  If not, just exit
		CLR	TOC		; TOCs rollover
XTIME		RTS			; Return from TIME

;*************************************************************************************
; BLINK - Update LED
; IF TOC is even, light LED else turn off LED
;*************************************************************************************
BLINK		EQU	*		; Update LED
		LDA	TOC		; If even, LSB will be zero
		LSRA			; Shift LSB to carry
		BCS	LEDOFF		; If not, turn off LED
		BSET	LED,PORTA	; Turn on LED
		BRA	XBLINK		; Then exit
LEDOFF		BCLR	LED,PORTA	; Turn off LED
XBLINK		RTI			; Return from BLINK

;*************************************************************************************
; RTIF interrupt service routine
;*************************************************************************************
RTICNT	DEC	RTIFs		; On each RTIF decrement RTIFs
		BNE	ENDTOF	; Done if RTIFs not 0
		LDA	#3		; RTIFs counts 3 --> 0
		STA	RTIFs		; Reset TOFS count
		BSET	7,TIC		; Set MSB as a flag to MAIN
ENDTOF	BSET	RTIFR,TSCR	; Clear RTIF flag
AnRTI		RTI			; Return from RTIF interrupt

UNUSED	EQU	AnRTI		; Use RTI at AnRTI for unused interrupts to just return



;*************************************************************************************
; Interrupt and reset vectors
;*************************************************************************************
		ORG	Vectors		; Start of vector area
TIMVEC	FDB	RTICNT	; Count RTIFs 3/TIC
IRQVEC	FDB	START		; Not used
SWIVEC	FDB	START		; Not used
RESETV	FDB	START		; Beginning of program on reset
