; This program is an example of using the real-time interrupt and timer overflow interrupt on the J1A
; to produce a low-frequency PWM signal on a general-purpose I/O pin.  The value of DesiredPWM
; is arbitrarily set to $80.  Normally, the value of DesiredPWM would be generated by some other
; part of the main program.

#INCLUDE 'H705J1A.FRK'        ; Include the equates for the 
					; HC705J1A so all labels can be found. 
						

Percent100	EQU	$FF		; Desired PWM value for 100% duty
PWM		EQU	PA7		; PWM output on port A bit 7
					; Update the Desired PWM variable

		ORG	RAM		; Start of on chip RAM

DesiredPWM 	RMB	1		; Desired PWM duty cycle...
; expressed as the numerator of DesiredPWM/255
; 0 = continuous low	255 = continuous high

PWMCoarse	RMB	1		; Number of TOF interrupts...	
; before we start to compare PWMFine to value in the TCR.

PWMFine	RMB	1		; When TCR matches PWMFine,...
					; the PWM is set low.
; PWMFine is derived from the lower 4-bits of Desired PWM.
; These 4-bits are placed in the upper 4-bits of PWMFine.

VarEnd	EQU	*

;***********************************************************************
;
		ORG	EPROM		; Start of on chip ROM/EPROM

START		EQU	*
		RSP			; Reset the stack pointer
		CLR	PORTA		; Set Port A outputs to all 0's
		LDA	#$FF		; Make all Port A's pins outputs
		STA	DDRA
					; Clear out all of RAM
		LDX	#RAM		; Point to the start of RAM
ClrLoop	CLR	,X		; Clear a byte
		INCX			; Point to the next location
					; Cleared the last location?
		BNE	ClrLoop	; No, Continue to clear RAM

		LDA 	#$80		; Corresponds to 50% (128/255)
		STA	DesiredPWM	; Establish a PWM duty cycle
		LDA	#$1C		; Clear timer ints...
		STA	TSCR		; and enable RTIF interrupt
		CLI			; Enable interrupts
		BRA	*		; Infinite loop, PWM uses interrupts
;***********************************************************************
; RTI Ssets perios. @2MHz & RT1:RT0 = 0:0, period = 8.192 ms or about
; 122 Hz
TimerInt		EQU	*
		BRSET	TOF,TSCR,TOFInt	; TOF interrupt?
		BRSET	RTIF,TSCR,RTIInt	; RTI interrupt?
		RTI
;***********************************************************************
; TOF interrupt response - Decrement PWMCoarse, when 0...
; Compare PWMFine to TCR.  When TCR passes PWMFine clear PWM
; output pin and disable further TOF.  RTI re-enables.

TOFInt	EQU	*
		DEC	PWMCoarse	; Is PWMCoarse=0
		BNE	ExitTOF	; No. Clear TOF and return
TOFInt1	LDA	PWMFine	; To compare to upper 4 of TCR
CmpMore	CMPA	TCR		; Loop till PWMFine <= TCR
		BCLR	PWM,PORTA	; Set the PWM output low (0V)
		BCLR	TOIE,TSCR	; Disable the TOF Interrupt
ExitTOF	BSET	TOFR,TSCR	; Reset the TOF Interrupt Flag
		RTI			; Return to the main program

;***********************************************************************
; RTIF interrupt response - Set PWM out pin high, and enable TOF.  Make 
; PWMCoarse & PWMFine from DesiredPWM
;
RTIInt	EQU	*
		BSET	RTIFR,TSCR	; Clear the RT Interrupt Flag
		LDA	DesiredPWM	; Get desired PWM level. =0?
		BEQ	RTIInt2	; Yes. Leave PWM output low
		BSET	PWM,PORTA	; No. Set PWM output high
		CMPA	#Percent100	; Desired PWM level 100%?
		BEQ	RTIInt2	; Yes. Leave PWM output high
		CLRX			; No. Put upper 4-bits of DesiredPWM into 
		LSLA			; lower 4-bits of A and lower 4-bits of
		ROLX			; DesiredPWN into the upper 4-bits of x.
		LSLA
		ROLX
		LSLA
		ROLX
		LSLA
		ROLX
		STA	PWMFine	; Save result into PWMFine.
RTIInt1	STX	PWMCoarse	; Save result into PWMCoarse.
		BEQ 	TOFInt1	; If PWMCoarse=0, go to 2nd
					; half of the TOF routine
		BSET	TOFR,TSCR	; Clear Timer Overflow Flag
		BSET	TOIE,TSCR	; re-enable the TOF interrupt
RTIInt2	RTI			; return from RTIF interrupt

		ORG	Vectors	; interrupt/reset vectors
		FDB	TimerInt	; timer interrupt routine
		FDB	Start		; IRQ vector (not used)
		FDB	Start		; SWI vector (not used)
		FDB	Start		; Reset vector


