******************************************************************************
******************************************************************************
*                                                                            *
*              Subroutine DelaymS - Delay for whole number of milliseconds   *
*                                                                            *
******************************************************************************
*                                                                            *
* File Name: delayms.SUB                     Copyright (c) Motorola 1994     *
*                                                                            *
* Curent Revision: 1.00                                                      *
* Current Release Level: PA                                                  *
* Current Revision Release Date: 04/20/95                                    *
*                                                                            *
* Current Release Written By: David Yoder                                    *
*                             Motorola CMCU Applications - Austin, Texas     *
*                                                                            *
* Assembled Under: IASM05 (P&E Microcomputer Systems, Inc.)    Ver.: 3.02m   *
*                                                                            *
* Documentation File Name: n/a                    Revision: n/a              *
*                                                                            *
* Brief Description of Module Purpose:                                       *
*             The routine delays for a whole number of milliseconds.         *
*             Minimum = 1mS, Maximum = 256mS.                                *
*             Assumes an HC05 CPU with 4MHz crystal, 2MHz bus speed          *
*                                                                            *
* Part Family Software Module Works With:  HC05                              *
* Part Module Software Module Works With:  only CPU05 is required            *
* Part Module Software Module Tested With: HC705C8A                          *
*                                                                            *
* Calling Sequence: LDA #desired number of milliseconds                      *
*                   JSR DelaymS_Body                                         *
*                                                                            *
* Entry Label:                    DelaymS_Body                               *
*  Module Size (Bytes):           17T bytes                                  *
*  Stack Space Used (Bytes):      2T bytes (1 jsr stack frame)               *
*  RAM Used (Bytes):              0                                          *
*  Worst Case Execution (Cycles): 512006T cycles (with $00 in Acc)           *
*  Entry Conditions:              # of mS to delay in Acc                    *
*  Part Resources Needed:         CPU05 running at 2.00MHz bus frequency     *
*  External Variables Used:       none                                       *
*  Subroutines Used:              none                                       *
*  Number of Exit Points:         1                                          *
*    Exit Label:                  DelaymS015                                 *
*      Exit Conditions:           Acc = 0 after being decremented            *
*                                                                            *
*                                                                            *
* Full Functional Description of Module Design:                              *
*                                                                            *
* This routine delays operation for a whole number of milliseconds.          *
* The number of milliseconds to delay is passed in the accumulator           *
* The routine alters Acc, X and CCR.                                         *
* A 4MHz clock (2MHz bus) is assumed.                                        *
* The smallest delay is 2011 cycles which occurs when Acc = 1. (1mS)         *
* The largest delay is 512006 cycles which occurs when Acc = 0. (256mS)      *
*                                                                            *
* Please note that passing 0 will NOT result in zero delay, but 256mS delay. *
*                                                                            *
*                                                                            *
* The number of milliseconds to delay is passed in the accumulator. The      *
* routine is formed by two loops. The inner loop (DelaymS020) executes in    *
* 1989 cycles. The outer loop executes once for each millisecond and adds    *
* 11 bus cyces each time through the loop. This creates 2000 cycles for      *
* each millisecond of delay. The RTS used to exit the routine add 06 bus     *
* cycles to the total time. By my convention, the JSR is not counted in      *
* this routine, but in the calling routine.                                  *
*                                                                            *
* The exact number of cycles for this routine to execute may be calculated   *
* from:                                                                      *
*                cycles = 11 + ( Acc * 2000 )                                *
*                                                                            *
* Upon exit, the accumulator and index register will be zero.                *
*                                                                            *
*                                                                            *
******************************************************************************
*                                                                            *
* Update History:                                                            *
* Rev:      Author:     Date:        Description of Change:                  *
* ----      -------     -----        ----------------------                  *                                      *
* PA 1.0    Yoder       04/20/95     Original Release                        *
*                                                                            *
******************************************************************************
******************************************************************************
*                                                                            *
* Motorola reserves the right to make changes without further notice to any  *
* product herein to improve reliability, function, or design. Motorola does  *
* not assume any liability arising out of the application or use of any      *
* product, circuit, or software described herein; neither does it convey any *
* license under its patent rights nor the rights of others. Motorola         *
* products are not designed, intended, or authorized for use as components   *
* in systems intended for surgical implant into the body, or other           *
* applications intended to support life, or for any other application in     *
* which the failure of the Motorola product could create a situation where   *
* personal injury or death may occur. Should Buyer purchase or use Motorola  *
* products for any such intended or unauthorized application, Buyer shall    *
* indemnify and hold Motorola and its officers, employees, subsidiaries,     *
* affiliatees, and distributors harmless against all claims, costs, damages, *
* and expenses, and reasonable attorney fees arising out of, directly or     *
* indirectly, any claim of personal injury or death associated with such     *
* unintended or unauthorized use, even if such claim alleges that Motorola   *
* was negligent regarding the design or manufacture of the part. Motorola    *
* and the Motorola Logo are registered trademarks of Motorola Inc.           *
*                                                                            *
******************************************************************************
******************************************************************************
.PAGE
******************************************************************************
*                                                                            *
* Delay for XmS                                                              *
*                                                                            *
* This calls the 1mS delay routine below.                                    *
* Number of mS in passed through the accumulator.                            *
*                                                                            *
******************************************************************************
DelaymS_Body    equ     *
DelaymS010      jsr     DelaymS020      ;Jump to 1mS delay counter
                brn     DelaymS010      ;burn 3 bus cycles
                nop                     ;burn 2 bus cycles
                deca                    ;decrement # of mS to delay
                bne     DelaymS010      ;branch if not done
DelaymS015      rts                     ;all done - return to calling routine
******************************************************************************
*                                                                            *
* Delay for 1mS                                                              *
*                                                                            *
* Takes 1989 cycles to execute.                                              *
*                                                                            *
******************************************************************************
DelaymS020      ldx     #$F7            ;Load delay into X
DelaymS030      decx                    ;Decrement delay
                nop                     ;burn 2 bus cycles
                bne     DelaymS030      ;Branch in not done
                rts


