

**************************************************

*MC68HC705KJ1 Example Development Project        *

*A home TV remote control to select TV program   *

*This example uses a 4X4 keypad, IRLED.          *
* Written by Chun-Hung Wong                      *
**************************************************



* Register Equates

PORTA      EQU    $00

PORTB      EQU    $01

DDRA       EQU    $04

DDRB       EQU    $05

TSCR       EQU    $08

TCR        EQU    $09

ISCR       EQU    $0A

PDRA       EQU    $10

PDRB       EQU    $11

EPROG      EQU    $18

MOR        EQU    $07F1



*RAM EQUATES

           ORG     $C0    ;$C0 IS THE START OF RAM

TEMPA      RMB     1

TEMPX      RMB     1

COUNT      RMB     1

OFFSET     RMB     1

SHIFT      RMB     1

TEMPIN     RMB     1

TEMPOUT    RMB     1

VALUE      RMB     1

DEFULT0    RMB     1

TEMPOFF    RMB     1



           ORG    $300

*$0100 IS THE START OF ERPROM IN THE 68HC705KJ1

INIT       RSP

           CLRA

           STA    PORTA

           CLRA

           STA    PORTB

           LDA    #$F0       ;PA0 - PA3 FOR INPUT, PA4 - PA7 FOR OUTPUT

           STA    DDRA

           LDA    #$0C       ;PB2,PB3 FOR OUTPUT

           STA    DDRB

           LDA    #$F0       ;ABLED PA0 - PA3 PULLDOWN RESISTERS

           STA    PDRA

           LDA    #$C        ;TO SET THE MOR BITS

           STA    MOR        ;-,-,-,-,*SWPDI,*PIRQ,-,-



********************************************************

*MAIN - BEGINNING OF THE MAIN LOOP                     *

*       LOOP IS EXCUTED ONCE EVERY 50 ms               *

********************************************************



MAIN       JSR     KYPAD

           LDA     TEMPOFF     ;TO ENSURE CORRECT KEY ENTRY

           CMP     #$00        ;THE TEMPOFF MUST LARGER THAN 0

           BLO     MAINLOOP

           JSR     DISPLAY     ;TO EMIT THE SIGNAL

           JSR     DLY50       ;TIME SEPARATION BETWEEN ENTRY

           CLRA                ;TO CLEAR THE OFFSET FOR EACH LOOPING

           STA     TEMPOFF

MAINLOOP   BRA     MAIN





* TO DETECT WHETHER THERE IS A KEY OR NOT

KYPAD      LDA     #$F0         ;TO TURN ON ALL THE COLS

           STA     PORTA

           LDA     #$80         ;ACTIVE *IRQF FOR INTERRUPT

           STA     ISCR

           LDA     PORTA        ;TO TEST WHETHER THE VOLUME KEY IS STILL PRESSED

           AND     #$08         ;THE VOLUMN KEY IS ON PA3

           TSTA                 ;THE KEY STILL PRESSED?

           BNE     ANYK         ;IF YES, SKIP THE STOP MODE

           STOP                 ;STOP MODE

ANYK       LDA     PORTA        ;READ ROWS IN LOWER 4

           AND     #$0F         ;MASK AWAY COLS

           BEQ     ANYK         ;EXIT IF NO KEY

           LDX     #$B4          ;POINTER TO LAST LINE IN KYTBL

KYLOOP     LDA     KYTBL,X      ;GET ROW/COL PATTERN

           STA     PORTA

           CMP     PORTA        ;TO FIND OUT THE CORRESPONDING KEY

           BEQ     FOUND        ;GO TO SUBROUTINE IF FOUND

           STX     TEMPIN       ;KEEP THE X UNCHANGED

           LDA     TEMPIN       ;DECREASE THE OFFSET ONCE PER LOOP

           SUB     #$C

           STA     TEMPOUT      ;TO TEMP STORE THE OFFSET VALUE

           LDX     TEMPOUT      ;TRANSFER THE OFFSET FROM ACC TO X

           CLRA                 ;ENSURE EMPTY ACC

           BPL     KYLOOP       ;LOOP UNTIL KEY IS DETECHED

           RTS



FOUND      STX     TEMPOFF      ;STORE THE OFFSET ONCE FOUND

           CLRX

           RTS



*DLY50 - SUBROUTINE TO DELAY 50 ms

DLY50      STA     TEMPA         ;DELAY FOR EXACTLY 50ms

           LDA     #$20

OUTLP      CLRX

INNRLP     DECX

           BNE     INNRLP

           DECA

           BNE     OUTLP

           LDA     TEMPA

           RTS



*keypad Correspondance Table

KYTBL      FCB     $81,'0','0','0','0','0','0','0','0','0','0'

           FCB     $07

           FCB     $82,'0','0','0','0','0','0','1','0','0','0'

           FCB     $07

           FCB     $84,'0','1','0','0','0','0','0','0','0','0'

           FCB     $07

           FCB     $88,'0','0','1','0','1','0','1','0','0','0'

           FCB     $07

           FCB     $41,'0','0','0','0','1','0','0','0','0','0'

           FCB     $07

           FCB     $42,'0','0','0','0','1','0','1','0','0','0'

           FCB     $07

           FCB     $44,'0','1','0','0','1','0','0','0','0','0'

           FCB     $07

           FCB     $48,'0','0','1','0','0','0','1','0','0','0'

           FCB     $07

           FCB     $21,'0','0','0','0','0','1','0','0','0','0'

           FCB     $07

           FCB     $22,'0','0','0','0','0','1','1','0','0','0'

           FCB     $07

           FCB     $24,'0','0','1','0','0','1','1','0','0','0'

           FCB     $07

           FCB     $28,'0','0','1','0','0','1','0','0','0','0'

           FCB     $07

           FCB     $11,'0','0','0','0','1','1','0','0','0','0'

           FCB     $07

           FCB     $12,'0','0','0','0','1','1','1','0','0','0'

           FCB     $07

           FCB     $14,'0','0','1','0','0','0','0','0','0','0'

           FCB     $07

           FCB     $18,'0','0','1','0','1','0','0','0','0','0'

           FCB     $07

DISPLAY    LDA     #$04          ;REPEAT THE SIGNAL SEQUENCE FOR 4 TIMES

           STA     COUNT

OUTER      LDA     TEMPOFF       ;TO START THE EMISSION SEQUENCE FROM POINTED

           STA     OFFSET        ;TO IDENTIFY THE OFFSET ADDRESS AS FOUND ABOVE

           JSR     INITIAL       ;IT IS THE INITIAL SEQUENCE '1110001'

INNER      INC     OFFSET        ;INCREASE THE OFFSET ONE BY ONE

           LDX     OFFSET

           LDA     KYTBL,X       ;TO LOAD THE CORRESPONDING SIGANL '1' OR '0'

           CMP     '1'           ;IS IT '1'?

           BNE     TRY0          ;OTHERWISE, TRY '0'

           JSR     OUTPUT1       ;IF YES, OUTPUT  '1'

           BRA     INBACK        ;TEST THE NEXT ONE

TRY0       CMP     '0'           ;IS IT '0'?

           BNE     TRYOTHERS     ;OTHERWISE, TRY OTHERS

           JSR     OUTPUT0       ;IF YES, OUTPUT '0'

           BRA     INBACK        ;TEST THE NEXT ONE

TRYOTHERS  CMP     #$07          ;IS IT THE END?

           BEQ     EXIT          ;IF YES, EXIT

INBACK     BRA     INNER         ;REPEAT THE DETECTION

EXIT       JSR     SEPARAT       ;THERE SHOULD BE A SEPARATION BETWEEN SEQUENCES

           DEC     COUNT

           BNE     OUTER         ;REPEAT AGAIN UNTIL 4 TIMES

           RTS



INITIAL    JSR     OUTPUT1       ; TO DISPLAY THE INITIAL SEQUENCE(1110001)

           JSR     OUTPUT1

           JSR     OUTPUT1

           JSR     OUTPUT0

           JSR     OUTPUT0

           JSR     OUTPUT0

           JSR     OUTPUT1

           RTS



OUTPUT0    JSR     HEAD          ;TO DISPLAY THE HEAD WHICH IS SAME FOR BOTH '1' AND '0'

           LDX     #$AA          ;170 x 8 x 0.5 = 680 us

           STX     VALUE         ;READY FOR TIMING

           BCLR    2,PORTB       ;BCLR = '0'

           JSR     TIMING        ;JUMP TO SUBROUTING 'TIMING'

           RTS



OUTPUT1    JSR     HEAD          ;SAME AS OUTPUT0

           BCLR    2,PORTB

           LDX     #$FF          ;AS THE TAIL IS TOO LONG, IT IS CUT DOWN TO REPEAT

           STX     VALUE         ;TWICE AND ALSO A REMAINDER

           LDA     #$1           ;475 X 8 X 0.5 = 1.9 ms

OUTLOOP1   CMP     #$00          ;475 = 255 X 1 + 220( FF +DC )

           BEQ     REMAIN1       ;THE REMAINDER

           BCLR    2,PORTB

           JSR     TIMING

           DECA

           BRA     OUTLOOP1

REMAIN1    LDX     #$DC          ;IT IS ALSO '0'

           STX     VALUE

           BCLR    2,PORTB

           JSR     TIMING

           RTS



TIMING     LDX     VALUE          ;GET THE EXACT TIMEING NEEDED

L100       DECX

           NOP

           BNE     L100           ;LOOP UNITL TIME OUT

           RTS



SEPARAT    LDX     #$FF           ;THE SEPARATION BETWEEN SEQUENCES

           STX     VALUE

           LDA     #$19

OUTLOOP2   CMP     #$00

           BEQ     REMAIN2

           JSR     TIMING

           DECA

           BRA     OUTLOOP2        ; 6550 x 8 x 0.5 = 26.2 ms

REMAIN2    LDX     #$AF            ; 6550 = 25 X 255 + 175(FF X 19 + AF)

           STX     VALUE

           JSR     TIMING

           RTS



HEAD       LDX     #$12            ;TO REPEAT 18 TIMES, TOTAL 18 X 30us = 540us

L140U      BSET    2,PORTB         ;TOTAL 6 CYCLES TIME FOR BEING HIGH 1

           BSET    2,PORTB

           BSET    2,PORTB

           BSET    2,PORTB

           BSET    2,PORTB

           BSET    2,PORTB         ; TOTAL 6 X 2.5us = 15 us

           BCLR    2,PORTB

           BCLR    2,PORTB

           BCLR    2,PORTB

           BCLR    2,PORTB

           BCLR    2,PORTB         ;TOTAL 5 CYCLE TIME FOR BEING LOW

           DECX

           BNE     L140U           ; 5 X 2.5 + 6 X 0.5 =15.5

           RTS



EXITINT    RTI                     ;GET OUT OF THE INTERRUPT



           ORG     $07FE           ;RESET VECTOR ADDRESS

           FDB     INIT            ;RESET VECTOR

           ORG     $07F8           ;TO ENSURE WHEN KEY IS DETECTED AFTER THE STOP

           FDB     EXITINT         ;MODE, THE PROGRAM COULD PROMPTLY AND CORRECTLY

           ORG     $07FA           ;ESCAPE THE INTERRUPT TABLE

           FDB     EXITINT

           ORG     $07FC

           FDB     EXITINT

